from typing import Optional

from overhave.transport.http import BaseHttpClientSettings
from overhave.transport.http.gitlab_client.objects import TokenType


class OverhaveGitlabClientSettings(BaseHttpClientSettings):
    """ Settings for :class:`GitlabHttpClient`. """

    auth_token: Optional[str] = None
    token_type: TokenType

    class Config:
        env_prefix = "OVERHAVE_GITLAB_"



import abc
from functools import cached_property

from overhave.factory.base_factory import BaseOverhaveFactory, IOverhaveFactory
from overhave.factory.components.abstract_consumer import ITaskConsumerFactory
from overhave.factory.context import OverhavePublicationContext
from overhave.publication import IVersionPublisher, StashVersionPublisher
from overhave.publication.gitlab import GitlabVersionPublisher, TokenizerClient
from overhave.publication.objects import PublicationManagerType
from overhave.transport import GitlabHttpClient, PublicationTask, StashHttpClient


class BasePublicationFactoryException(Exception):
    """ Base exception for :class:`PublicationFactory`. """


class AuthTokenNotSpecifiedError(BasePublicationFactoryException):
    """ Exception for situation AuthToken env was not set. """


class UrlGitlabTokenizerNotScepifiedIfEnabled(BasePublicationFactoryException):
    """ Exception for situation tokenizer is enabled and url env was not set. """


class IPublicationFactory(IOverhaveFactory[OverhavePublicationContext], ITaskConsumerFactory[PublicationTask], abc.ABC):
    """ Abstract factory for Overhave publication application. """

    @property
    @abc.abstractmethod
    def publisher(self) -> IVersionPublisher:
        pass


class PublicationFactory(BaseOverhaveFactory[OverhavePublicationContext], IPublicationFactory):
    """ Factory for Overhave publication application. """

    context_cls = OverhavePublicationContext

    @cached_property
    def _stash_client(self) -> StashHttpClient:
        return StashHttpClient(self.context.client_settings)  # type: ignore

    @cached_property
    def _gitlab_client(self) -> GitlabHttpClient:
        return GitlabHttpClient(self.context.client_settings)  # type: ignore

    @cached_property
    def _stash_publisher(self) -> StashVersionPublisher:
        return StashVersionPublisher(
            file_settings=self.context.file_settings,
            project_settings=self.context.project_settings,
            feature_storage=self._feature_storage,
            scenario_storage=self._scenario_storage,
            test_run_storage=self._test_run_storage,
            draft_storage=self._draft_storage,
            file_manager=self._file_manager,
            stash_publisher_settings=self.context.publisher_settings,  # type: ignore
            client=self._stash_client,
        )

    @cached_property
    def _tokenizer_client(self) -> TokenizerClient:
        return TokenizerClient(self.context.tokenizer_client_settings)

    @cached_property
    def _gitlab_publisher(self) -> GitlabVersionPublisher:
        if not self._tokenizer_client._settings.enabled and self._gitlab_client._settings.auth_token is None:
            raise AuthTokenNotSpecifiedError("Please set correct auth_token!")
        if self._tokenizer_client._settings.enabled and self._tokenizer_client._settings.url is None:
            raise UrlGitlabTokenizerNotScepifiedIfEnabled("Please set correct url for gitlab_tokenizer!")
        return GitlabVersionPublisher(
            file_settings=self.context.file_settings,
            project_settings=self.context.project_settings,
            feature_storage=self._feature_storage,
            scenario_storage=self._scenario_storage,
            test_run_storage=self._test_run_storage,
            draft_storage=self._draft_storage,
            file_manager=self._file_manager,
            gitlab_publisher_settings=self.context.publisher_settings,  # type: ignore
            gitlab_client=self._gitlab_client,
            tokenizer_client=self._tokenizer_client,
        )

    @property
    def publisher(self) -> IVersionPublisher:
        if self.context.publication_settings.publication_manager_type is PublicationManagerType.GITLAB:
            return self._gitlab_publisher
        return self._stash_publisher

    def process_task(self, task: PublicationTask) -> None:
        return self.publisher.process_publish_task(task)



from typing import Any, Dict, Optional

from pydantic import validator
from yarl import URL

from overhave.transport.http import BaseHttpClientSettings


class TokenizerClientSettings(BaseHttpClientSettings):
    """ Important environments and settings for :class:`TokenizerClient`. """

    enabled: bool = False
    url: Optional[URL] = None  # type: ignore
    initiator: Optional[str] = None
    remote_key: Optional[str] = None
    remote_key_name: Optional[str] = None

    class Config:
        env_prefix = "OVERHAVE_GITLAB_TOKENIZER_"

    @validator("remote_key", "initiator", "remote_key_name")
    def validate_remote_key_and_initiator(cls, v: Optional[str], values: Dict[str, Any]) -> Optional[str]:
        if values["enabled"] and not isinstance(v, str):
            raise ValueError(
                "Please verify REMOTE_KEY, REMOTE_KEY_NAME and INITIATOR! Maybe you've forgotten about this env!"
            )
        return v



from pathlib import Path
from typing import Optional

from pydantic.datetime_parse import timedelta

from overhave.base_settings import BaseOverhavePrefix
from overhave.entities.authorization.strategies import AuthorizationStrategy


class OverhaveAdminSettings(BaseOverhavePrefix):
    """ Settings for Overhave Flask Admin customization. """

    # Path to custom index template. By default, contains Overhave project info.
    index_template_path: Optional[Path]

    # Enable testing with test execution consumer, based on Redis tasks. Enabled by default.
    # When disabled - all test runs will be executed with :class:`Threadpool`.
    # TODO: realise all other consumer actions with Threadpool for better customization.
    consumer_based: bool = True

    # Threadpool size for admin service
    threadpool_process_num: int = 5


class OverhaveAuthorizationSettings(BaseOverhavePrefix):
    """ Settings for Overhave authorization in components interface.

    Supports 3 strategies: SIMPLE, DEFAULT and LDAP.
    LDAP authorization uses group politics with administration group `admin_group`.
    SIMPLE and DEFAULT strategies use admin user that would be dynamically created at startup.
    """

    auth_strategy: AuthorizationStrategy = AuthorizationStrategy.SIMPLE
    admin_group: Optional[str] = None


class OverhaveLdapClientSettings(BaseOverhavePrefix):
    """ Settings for Overhave LDAP client for AuthorizationStrategy.LDAP strategy. """

    ldap_url: str  # for example: "ldap://mydomain.ru"
    ldap_domain: str  # for example: "domain\\"
    ldap_dn: str  # for example: "dc=example,dc=com"
    ldap_timeout: timedelta = timedelta(seconds=10)
